;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PIC16F676. This file contains the basic code               *
;   building blocks to build upon.                                    *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PIC data sheet for additional             *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    ZU676.asm                                           *
;    Date:                                                            *
;    File Version:   v1                                                 *
;                                                                     *
;    Author:       NickF                                                   *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files Required: P16F676.INC                                      *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;**********************************************************************

	list      p=16f676           ; list directive to define processor
	#include <p16F676.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT 

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.




;***** VARIABLE DEFINITIONS

	CBLOCK 0x20			;      

TenK,Thou,Hund,Tens,Ones,flags
count,t_zn,znk,cyc_tim,bri
NumH,NumL,TEMP1,TEMP0,znak,zn_dac						;NumH,L-HEX , d(n) - DEC 

		ENDC
;flags -  0.1 -  1.  3  :1 - ;2 - 
;3 -  - 
;flags -  2.3-  2.  4   
;flags -  4 -  3. 1-  , 0-  
;flags -  5.6 -  4.   2,6,10 .
;flags -  7 - "1"  "zn_dac"   U1
;znak - 7-  , znk -  1,2,3,4 
;bri -  
;count,t_zn -  
;cyc_tim -   TMR0
;NumH   ;A3*16+A2
;NumL    ;A1*16+A0
;share variables
;=B4*104 + B3*103 + B2*102 + B1*101 + B0*100
;=B4*10000 + B3*1000 + B2*100 + B1*10 + B0
;TenK     	B4
;Thou     	B3
;Hund     	B2
;Tens     	B1
;Ones		B0

;**********************************************************************
		ORG     0x000             ; processor reset vector
		goto    main              ; go to beginning of program

		ORG     0x004             ; interrupt vector location
		goto	obint
		org	0x10
segment		;   DEC ->  .
    CLRF   PCLATH
	ADDWF  PCL, F 
	dt	07E, 00C, 0B6, 09E, 0CC, 0DA, 0FA, 00E, 0FE, 0DE 
;-    -  
;	dt	0FC, 060, 0DA, 0F2, 066, 0B6, 0BE, 0E0, 0FE, 0F6
;-   -    
;*******************************************************************
main
		; init PORTA & SFR

		bcf 	STATUS,RP0 			;Bank 0
		clrf 	PORTA 				;Clear PORTA
		movlw 	07h 				;Set RA<2:0> to 
		movwf 	CMCON 				;comp-off digital I/O
		bsf 	STATUS,RP0 			;Bank 1
		clrf 	ANSEL 				;digital I/O
		clrf 	WPUA 				;  
		movlw 	07h 				;
		movwf 	ANSEL 				;Set PORTA  RA<3>  analog input
		movlw 	03fh
		movwf 	TRISA				;Set PORTA as input
		bsf		WPUA,4
		bsf		WPUA,5
		clrf	IOCA				;clear Interrupt-on-Change PORTA
		bsf 	IOCA,3				;    RA3	
		clrf 	PIE1 				;   		
;		movlw 	03h 				;Set POR and BOR 
;		movwf 	PCON 				;  Vdd

		;init PORTC
		clrf	PORTC
		bsf 	STATUS,RP0 			;Bank 1
		clrf 	TRISC 				;set PORTC as outputs
		bcf 	STATUS,RP0 			;Bank 0

; these first 4 instructions are not required if the internal oscillator is not use

		call    0x3FF             ; retrieve factory calibration value
		bsf     STATUS,RP0        ; set file register bank to 1 
		movwf   OSCCAL            ; update register with factory cal value 
		bcf     STATUS,RP0        ; set file register bank to 0

; ADC,TMR0
	banksel		ADCON1			
		movlw	030h			;    
		movwf 	ADCON1			;RC  ~ 8 us.
		clrf	TMR0			;  "0"
		movlw	02h				; - 
	banksel		OPTION_REG		;   1/8 
		movwf	OPTION_REG		; CLK  TMR0 - 8 ,   "T0CS"=0
;   256*8=2.048 .
		
; remaining code goes here
start
;  020h-04fh
		movlw	020h			;   
		movwf	FSR
	
c_ind	clrf	INDF			;      FSR
		incf	FSR,f			; FSR
		btfss	FSR,6			;FSR  40h? 
		goto	c_ind			;,     
;  
		movlw	6
		movwf	cyc_tim		;   5
		movwf	bri				; 
		movlw	080h
		movwf	zn_dac
		swapf	zn_dac,f
;	
start_adc
		banksel	PIE1			;     
		bsf		PIE1, ADIE 		;   
		banksel	ADCON0			;   ADCON0
		btfsc	flags,4			; 
		goto	tok				;,   tok
		movlw	081h			;  
		movwf 	ADCON0			;-AN0, Vref = VDD.
		goto	zap_adc
tok		movlw	0C9h			;  
		movwf 	ADCON0			;-AN2, Vref = 1.024V.
zap_adc		bsf		ADCON0,GO		; ADC	
		movlw	6
		movwf	cyc_tim			;   "TMR0'	
; 
		bcf 	STATUS,RP0 		;Bank 0
		bsf		INTCON,PEIE		;   
		bsf		INTCON,RAIE		;    RA3
		bsf		INTCON,T0IE		;    TMR0
		bsf		INTCON,GIE		; 
;   
		call	mult		; /     5
		call	HDF			; /  HEX -> DEC

		movlw	1
		movwf	znk			; 1  
st3
		movf	cyc_tim,f
		skpz
		goto	st3
		movlw	.6
		movwf	cyc_tim
		movlw	020h		;   
		addwf	znk,w
		movwf	FSR
		movf	INDF,w		;      
		call	segment		;/    7  
		movwf	znak		;akk->znak
		call	write_zn	;/    
;      
		call	button		;  ,   /  
;   ,     ,    
		incf	znk,1
		movlw	5
		subwf	znk,0
		SKPZ
		goto	st3
		goto	start_adc
;/     LED 
;  /   znak 
;7-      
write_zn
		clrf	t_zn
		movf	znk,w		; "znk"  "w"
		movwf	TEMP0
		bsf		STATUS,C		
shi		rrf		t_zn,f		;     
		decfsz	TEMP0,f		;     
		goto	shi
		movf	t_zn,w
		xorwf 	zn_dac,w	;  "zn_dac"  "t_zn"
;*******************************
;		xorlw	0f0h		; ,     
;*******************************
		movwf	TEMP1		;    "zn_dac"
		movf	znak,w
;*******************************
		xorlw	0ffh		; ,     
;*******************************
		movwf	TEMP0		;    "znak"
		call	shi_16
		nop
		call	pause
		return
pause
		movlw	.105
		movwf	TEMP0
pa		decf	TEMP0,f		
		skpz
		goto	pa 
		return
;/  
button	
	banksel		PORTA
		BTFSC	flags,3
		goto	bt2			;
		;bcf		PORTA,4		; B2  
		goto	bt3
bt2		;bsf		PORTA,4
bt3		BTFSC	flags,4
		goto	bt5	
		;bcf		PORTA,5		; B3  
		goto	bt6
bt5		;bsf		PORTA,5
bt6		return
;  74164
RegZero
;/  
		clrf	TEMP0
		clrf	TEMP1
		call	shi_16
; /
		return
;/  
obint
	banksel		INTCON
		movwf	05eh
		movf	STATUS,w
		movwf	05fh				;
		bcf		INTCON,GIE			;  
		btfss	PIR1,ADIF			;  
		goto	int_next			;   
int_ADC
		bcf		PIR1,ADIF			;  ADIF
	banksel		ADRESL
		movf	ADRESL,w
		movwf	NumL				; . 
	banksel		ADRESH
		movf	ADRESH,w
		movwf	NumH				; . 
		goto	isr
int_next
	banksel		INTCON							
		btfss	INTCON,RAIF			;   	
		goto	it					;   ,   
		bcf		INTCON,RAIF
		movf	znk,w					
		movwf	flags			;  
		goto	isr
it		btfss	INTCON,T0IF		;
		goto	int_err			;   
		bcf		INTCON,T0IF		;  T0IF
		decf	cyc_tim,f		;   "TMR0"
		bsf		INTCON,T0IE		;   "TMR0"
isr
		bsf		INTCON,GIE		; 
int_err							; 
		movf	05fh, w			; "STATUS"
		movwf	STATUS
		movf	05eh, w				; "W"
		retfie                    	; return from interrupt
;/    "input" / 
mult					
;copy accumulator to temporary
		movf	NumH,w
		movwf	TEMP1
		movf	NumL,w
		movwf	TEMP0
;/  10- HEX-  5
;     Input: NumL .. NumH, 10 bits
;    Output: NumL .. NumH, 13 bits
; Code size: 16 instructions
; NumH_NumL*4	
		clrc
		rlf		TEMP0,f
		rlf		TEMP1,f
		rlf		TEMP0,f
		rlf		TEMP1,f	
;  NumH_NumL   TEMP1_TEMP0 
		movf	TEMP0,w
		addwf	NumL,f
		movf	TEMP1,w
		skpnc
		incfsz	TEMP1,w
		addwf	NumH,f
		return
; /
HDF			
		nop
HexDecFast
		swapf	NumH,w	;w  = A2*16+A3
        andlw   0x0F     ;w  = A3		*** PERSONALLY, I'D REPLACE THESE 2
        addlw   0xF0	;w  = A3-16	*** LINES WITH "IORLW b'11110000B' " -AW
        movwf   Thou	;B3 = A3-16
        addwf   Thou,f	;B3 = 2*(A3-16) = 2A3 - 32
        addlw   .226	;w  = A3-16 - 30 = A3-46
        movwf   Hund	;B2 = A3-46
        addlw   .50	;w  = A3-46 + 50 = A3+4
        movwf   Ones	;B0 = A3+4

        movf    NumH,w	;w  = A3*16+A2
        andlw   0x0F	;w  = A2
        addwf   Hund,f	;B2 = A3-46 + A2 = A3+A2-46
        addwf   Hund,f	;B2 = A3+A2-46  + A2 = A3+2A2-46
        addwf   Ones,f	;B0 = A3+4 + A2 = A3+A2+4
        addlw   .233	;w  = A2 - 23
        movwf   Tens	;B1 = A2-23
        addwf   Tens,f	;B1 = 2*(A2-23)
        addwf   Tens,f	;B1 = 3*(A2-23) = 3A2-69 (Doh! thanks NG)

        swapf   NumL,w	;w  = A0*16+A1
        andlw   0x0F	;w  = A1
        addwf   Tens,f	;B1 = 3A2-69 + A1 = 3A2+A1-69 range -69...-9
        addwf   Ones,f	;B0 = A3+A2+4 + A1 = A3+A2+A1+4 and Carry = 0 (thanks NG)

        rlf     Tens,f	;B1 = 2*(3A2+A1-69) + C = 6A2+2A1-138 and Carry is now 1 as tens register had to be negitive
        rlf     Ones,f	;B0 = 2*(A3+A2+A1+4) + C = 2A3+2A2+2A1+9 (+9 not +8 due to the carry from prev line, Thanks NG)
        comf    Ones,f	;B0 = ~(2A3+2A2+2A1+9) = -2A3-2A2-2A1-10 (ones complement plus 1 is twos complement. Thanks SD)
;;Nikolai Golovchenko [golovchenko at MAIL.RU] says: comf can be regarded like:
;;      comf Ones, f
;;      incf Ones, f
;;      decf Ones, f
;;First two instructions make up negation. So,
;;Ones  = -1 * Ones - 1 
;;      = - 2 * (A3 + A2 + A1) - 9 - 1 
;;      = - 2 * (A3 + A2 + A1) - 10
        rlf     Ones,f	;B0 = 2*(-2A3-2A2-2A1-10) = -4A3-4A2-4A1-20

        movf    NumL,w	;w  = A1*16+A0
        andlw   0x0F	;w  = A0
        addwf   Ones,f	;B0 = -4A3-4A2-4A1-20 + A0 = A0-4(A3+A2+A1)-20 range -215...-5 Carry=0
        rlf     Thou,f	;B3 = 2*(2A3 - 32) = 4A3 - 64

        movlw   0x07	;w  = 7
        movwf   TenK	;B4 = 7

;B0 = A0-4(A3+A2+A1)-20	;-5...-200
;B1 = 6A2+2A1-138	;-18...-138
;B2 = A3+2A2-46		;-1...-46
;B3 = 4A3-64		;-4...-64
;B4 = 7			;7
; At this point, the original number is
; equal to TenK*10000+Thou*1000+Hund*100+Tens*10+Ones 
; if those entities are regarded as two's compliment 
; binary.  To be precise, all of them are negative 
; except TenK.  Now the number needs to be normal- 
; ized, but this can all be done with simple byte 
; arithmetic.

        movlw   .10	;w  = 10
Lb1:			;do
        addwf   Ones,f	; B0 += 10
        decf    Tens,f	; B1 -= 1
        btfss   3,0
	;skip no carry
         goto   Lb1	; while B0 < 0
	;jmp carry
Lb2:			;do
        addwf   Tens,f	; B1 += 10
        decf    Hund,f	; B2 -= 1
        btfss   3,0
         goto   Lb2	; while B1 < 0
Lb3:			;do
        addwf   Hund,f	; B2 += 10
        decf    Thou,f	; B3 -= 1
        btfss   3,0
         goto   Lb3	; while B2 < 0
Lb4:			;do
        addwf   Thou,f	; B3 += 10
        decf    TenK,f	; B4 -= 1
        btfss   3,0
         goto   Lb4	; while B3 < 0
        retlw   0

;/  16-   16 
;  "TEMP0",     "TEMP1"
shi_16
		movlw	.16
		movwf	count		;   
per
		clrc	
		rrf		TEMP0,f
		rrf		TEMP1,f
		skpnc
		goto	seg1		; =1
		bcf		PORTC,2		; =0
		goto	seg0
seg1	bsf		PORTC,2
seg0	bsf		PORTC,1		; 
		nop
		nop
		bcf		PORTC,1		; 
		DECFSZ	count,f		;  
		goto	per			;    	
		return

;      RC 
			ORG   0x3FF 
           retlw   .128

; initialize eeprom locations
		
		ORG	0x2100
		DE	0x00, 0x01, 0x02, 0x03

		END                 ; directive 'end of program'
